#!/usr/bin/env ruby

#---
# Copyright 2003-2013 by Jim Weirich (jim.weirich@gmail.com).
# All rights reserved.

# Permission is granted for use, copying, modification, distribution,
# and distribution of modified versions of this work as long as the
# above copyright notice is included.
#+++

require 'flexmock/symbol_extensions'
require 'flexmock/object_extensions'
require 'flexmock/class_extensions'

require 'flexmock/core'

require 'flexmock/default_framework_adapter'
require 'flexmock/expectation_director'
require 'flexmock/expectation'
require 'flexmock/argument_matchers'
require 'flexmock/argument_types'
require 'flexmock/validators'
require 'flexmock/recorder'
require 'flexmock/mock_container'
require 'flexmock/partial_mock'
require 'flexmock/undefined'
require 'flexmock/deprecated_methods'
require 'flexmock/extensions/active_record_model'
